#!/usr/bin/env python

import spss, spssaux, spssdata, random
#*Script acts on active data set .

# Example of use within a PROGRAM block:
# GET FILE='C:\Program Files\SPSS\employee data.sav'.
# BEGIN PROGRAM.
# import findEmptyVars as fev
# fev.findEmptyVars()
# END PROGRAM.

__author__  =  'Raynald Levesque raynald@spsstools.net'
__version__ =  '1.0'

def createSampleFiles():
    """createSampleFiles() creates a file where some vars
        have no values across all cases
        have the same value across all cases
        have some missing values (note that long strings never have missing values)
    The program defines macro variable !listEmptyVars containing the list of
    vars with only one or no values across cases. This can be used to delete
    these variables when deemed appropriate.
    """
    spss.Submit(r"""
    CD "C:\temp\".
    GET FILE="C:\Program Files\SPSS\Employee data.sav".
    * create an empty string var, and one containing 'python' in all cases.
    STRING var2(A5) var3(A6).
    COMPUTE var3 = 'python'.
    * Create 3 empty numeric variables
    * num1 contains only SYSMIS values,
    * num2 contains only 0
    * num3 contains 10 in 4 cases, SYSMIS in other cases.
    NUMERIC num1 num2.
    COMPUTE num2=0.
    COMPUTE num3=10.
    IF $CASENUM<5 num2=$SYSMIS.
    SAVE OUTFILE="original data file.sav".""")

def findEmptyVars(deleteEmpty=False, greedy=False):
    """findEmptyVars() finds the list of empty vars in the active dataset. A variable is
    assumed to be empty if it contains
    - the same value across all cases (applies to strings and numeric vars),or
    - some user and/or system missing values as well as a single non missing value contained in
    one or more cases (when greedy=True, this is considered an empty variable)
    """

    varDict = spssaux.VariableDict()
    vlist = ['%s' % (var) for var in varDict.variables]
    vlist_Min = ['%s_Min' % (var) for var in varDict.variables]
    vlist_Max = ['%s_Max' % (var) for var in varDict.variables]
    vlist_Mis = ['%s_Mis' % (var) for var in varDict.variables]
    vstring = ' '.join(vlist)
    vstring_Min = ' '.join(vlist_Min)
    vstring_Max = ' '.join(vlist_Max)
    vstring_Mis = ' '.join(vlist_Mis)
    nobreak = 'T' + str(random.uniform(0, 1))

    # Ensure current dataset has a name
    dsname = spssaux.GetActiveDatasetName()
    if dsname is None:
        dsname = "T" + str(random.random())
        spss.Submit("DATASET NAME " + dsname)

    aggdsname = "T" + str(random.random())

    cmd = r"""
    SET MPRINT=YES /PRINTBACK=YES.
    SPLIT FILE OFF.
    COMPUTE %(nobreak)s =1.
    DO REPEAT v_Mis = %(vstring_Mis)s /var=%(vstring)s.
      COMPUTE v_Mis = MISSING(var).
    END REPEAT PRINT.
    DATASET DECLARE %(aggdsname)s.
    AGGREGATE OUTFILE=%(aggdsname)s
            /BREAK=%(nobreak)s
            /%(vstring_Max)s = MAX(%(vstring)s)
            /%(vstring_Min)s = MIN(%(vstring)s)
            /%(vstring_Mis)s = SUM(%(vstring_Mis)s)
            /nbCases = N.
    DATASET ACTIVATE %(aggdsname)s.
     """ % locals()
    try:
        spss.Submit(cmd)
    except:
        print cmd
        raise

    # Read aggregate file and determine which vars are empty
    data = spssdata.Spssdata()
    listEmptyVars = []
    for row in data:
        nbVars = len(vlist)
        print '\n************'
        print 'Info on vars that are empty or have a single value follow.'
        print '(The macro !listEmptyVars contains the list of these var names)\n'
        for idx in range(1,nbVars+1):
            nbMissing = row[idx + 2*nbVars] #includes user missing as well as sysmis
            nbCases = row[-1]
            if row[idx] == row[idx + nbVars]:
                if nbMissing == 0 or nbMissing == nbCases or greedy == True:
                    varname = data.namelist[idx][:-4]
                    print varname, '(',varDict.VariableFormat(idx),')', '\texisting value=',row[idx],\
                    '\t# cases with missing values=%u out of %u' % (row[idx + 2*nbVars], row[-1])
                    listEmptyVars.append(varname)
        data.close
        del data, row

    # define macro containing list of these variables
    print
    v_Empty = ''
    if deleteEmpty and listEmptyVars:
        v_Empty = ' '.join(listEmptyVars)
        spss.Submit(r"""
            DATASET ACTIVATE %(dsname)s.
            DELETE VAR %(vstring_Mis)s %(nobreak)s.
            DELETE VAR %(v_Empty)s.""" % locals())
    if v_Empty:
        spss.SetMacroValue('!listEmptyVars',v_Empty)


if __name__ == '__main__':
    # Test that the function works
    createSampleFiles()
    spss.Submit(r"""
       SET MPRINT=YES /PRINTBACK=YES.
       CD "C:/temp/".
       GET FILE='original data file.sav'. """)
    findEmptyVars(deleteEmpty=True, greedy=False)
    spss.Submit("LIST /CASES=FROM 1 TO 5.")
    #findEmptyVars(deleteEmpty=True, greedy=True)
